# Text widget procedure module.
# Rick Smereka, Copyright (C) 2004-2005.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, get a copy via the Internet at
# http://gnu.org/copyleft/gpl.html or write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA

# You can contact the author via email at rsmereka@future-lab.com

# Initial tcl version. Feb/2004, Rick Smereka
proc gotoline {tw row {col 0}} {
   # Goto a specific line and column
   # (col defaults to zero). Adjusts
   # the view of the text widget.
   # Returns 1 upon success, 0 otherwise.
   set lines [textlines $tw]

   if {[string is integer -strict $row] && $row >= 1 && $row <= $lines} {
      $tw mark set insert $row.$col
      $tw see $row.$col
      return 1
   }

   tk_messageBox -title "gotoline" -icon error -parent $tw \
      -message [format "Bad line number '%1\$s'\nMust be from 1 to %2\$d" \
      $row $lines]
   return 0
}

proc lineno {tw indx} {
   # Return the line number corresponding
   # to 'indx' upon success, 0
   # otherwise.
   set line 0
   set mes [$tw index $indx]

   if {![ll_word $mes line 1 .]} {
      set line 0
      return $line
   }

   return $line
}

proc textlines {tw} {
   # Determine the total number of lines.
   # Returns the number of lines upon success,
   # 0 otherwise.
   set lines 0
   set mes [$tw index {end - 1 char}]

   if {![ll_word $mes lines 1 .]} {
      set lines 0
      return $lines
   }

   return $lines
}
